/*
 * (C) Copyright 1996-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

/// @author Baudouin Raoult
/// @author Tiago Quintino
/// @date   December 2016

#ifndef UTIL_LIBOOPS_H_
#define UTIL_LIBOOPS_H_

#include <memory>
#include <string>

#include "eckit/system/Library.h"

namespace oops {

// -----------------------------------------------------------------------------

class LibOOPS : public eckit::system::Library {
 public:
  LibOOPS();

  ~LibOOPS();

  static LibOOPS& instance();

  virtual eckit::Channel& infoChannel() const;
  eckit::Channel& debugChannel() const override;

  virtual eckit::Channel& traceChannel() const;
  virtual eckit::Channel& statsChannel() const;
  virtual eckit::Channel& testChannel() const;

  void finalise();

 protected:
  const void* addr() const override;

  std::string version() const override;

  std::string gitsha1(unsigned int count) const override;

  mutable std::unique_ptr<eckit::Channel> infoChannel_;
  mutable std::unique_ptr<eckit::Channel> debugChannel_;

  mutable std::unique_ptr<eckit::Channel> traceChannel_;
  mutable std::unique_ptr<eckit::Channel> statsChannel_;
  mutable std::unique_ptr<eckit::Channel> testChannel_;

  size_t rank_;
  bool debug_;
  std::string predebug_;
  bool trace_;
  std::string pretrace_;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // UTIL_LIBOOPS_H_
